/*
 * Compilatori/preprocessori consigliati:
 * Linux:   g++
 * Windows: MinGW
*/

#ifdef __linux__
  #define os 1
#elif _WIN32
  #define os 2
  #include <time.h>
#else
  #define os 0
#endif

#include <iostream>
using namespace std;
#include <fstream>
#include <SDL/SDL.h>
#include <SDL/SDL_gfxPrimitives.h>
#include <SDL/SDL_mixer.h>
#include "funzioni.cpp"

char versione[15]; /* Numero di versione del programma */
char ROOTDIR[256]; /* Cartella in cui si trovano i file di configurazione di ppong */
SDL_Surface* schermo = NULL;
Uint8 *tasto = SDL_GetKeyState(NULL); /* Controlla keyinput da tastiera */
double lastframe = time(NULL); /* vedere fps() */
/* Numero di blocchi nello schermo, in larghezza e altezza */
const unsigned short width = 16;
const unsigned short height = 24;
/* Dimensioni fisiche dei blocchi, in pixel */
const unsigned short largBlock = 640 / width;
const unsigned short altBlock = 480 / height;
unsigned char block[width][height]; /* Array coi blocchi */
#include "tipi.cpp"
opt opzione;
giocatore player;
#include "info.cpp"
Mix_Music *musica = NULL;
Mix_Chunk *effetto[4] = { NULL };
#include "classi.cpp"
proietto palla;

unsigned int lvl; /* Livello corrente */
fstream filelvl; /* File con gli schemi per i livelli - vedere livelli.cpp per la descrizione */
char title[64]; /* Titolo-nome del livello */

#include "salva.cpp"
#include "menu.cpp"
#include "livelli.cpp"
#include "gameover.cpp"
#include "gioca.cpp"
#include "opzioni.cpp"

int main(int argc, char *argv[])
{
  /* NUMERO DI VERSIONE DEL PROGRAMMA */
  strcpy(versione, "0.99");

  /* Imposta variabili d'ambiente e altro a seconda del sistema operativo
   * vedere le direttive del preprocessore all'inizio di questo file */
  switch (os)
  {
    case 1:
      strcpy(ROOTDIR, getenv("HOME"));
      strcat(ROOTDIR, "/.ppong/");
      cout << "[I] Linux operating system detected\n" << endl;
      break;
    case 2:
      strcpy(ROOTDIR, getenv("USERPROFILE"));
      strcat(ROOTDIR, "\\ppong\\");
      clog << "[W] Windows operating system detected. Some features may not work!\n";
      break;
    default: cerr << "[E] Unable to detect operating system: some features may not be available!\n"; break;
  }

  srand(time(NULL));
  Uint32 SCHERMOINTERO = 0;
  Uint32 DOPPIO = SDL_DOUBLEBUF;
  /* Impostazione delle opzioni di default */
  opzione.difficolta = 2; /* Ammessi 1, 2 e 3 in ordine crescente */
  opzione.startLvl = 0;
  opzione.sonoro = true;
  opzione.autosave = false;
  opzione.volumeEffetti = MIX_MAX_VOLUME * 30 / 100;
  /* Impostazione delle opzioni salvate su file */
  if (! caricaOpzioni()) clog << "[W] Unable to load settings from file!\n";
  /* Impostazione delle opzioni passate da riga di comando */
  for (short v = 0; v < argc; v++)
  {
    if (argv[v][0] == '-')
    {
      switch (argv[v][1])
      {
        case 'l': opzione.startLvl = atoi(argv[v + 1]) - 1; break;
        case 'd': opzione.difficolta = atoi(argv[v + 1]); break;
        case 'm': opzione.sonoro = false; break;
        case 'h': infocmd(); return 0; break;
        case 'f': SCHERMOINTERO = SDL_FULLSCREEN; /* DOPPIO = 0 ;*/ break;
        case 'v': cout << "P Pong version " << versione << endl; return 0; break;
        default: clog << "[W] Parametro sconosciuto!\n"; break;
      }
    }
  }
  infolog();
  SDL_Init(SDL_INIT_VIDEO | SDL_INIT_AUDIO | SDL_INIT_TIMER);
  schermo = SDL_SetVideoMode(640, 480, 32, SDL_SWSURFACE | DOPPIO | SCHERMOINTERO);
  if (schermo == NULL)
  { cerr << "[E] Fatal error: Unable to SetVideoMode(). Run in a graphic environment!\n"; return 1; }

  if (! caricafont()) { cerr << "[E] Fatal error: unable to caricafont()!\n"; return 1; }

  if (opzione.sonoro)
  {
    if (Mix_OpenAudio(MIX_DEFAULT_FREQUENCY, MIX_DEFAULT_FORMAT, 2, 1024) == -1)
    { clog << "[W] Unable to Mix_OpenAudio()\n"; }
    musica = Mix_LoadMUS("data/background.ogg");
    if (musica == NULL) clog << "[W] Unable to Mix_LoadMUS()\n";

    if (! Mix_AllocateChannels(4)) clog << "[W] Unable to Mix_AllocateChannels()\n";
    clog << "[I] Errors in loading Mix_Chunk sound effects will not be notified\n" << endl;
    effetto[0] = Mix_LoadWAV("data/sound00.wav");
    effetto[1] = Mix_LoadWAV("data/sound01.wav");
    effetto[2] = Mix_LoadWAV("data/sound02.wav");
  }
  else
  {
    cout << "[I] Sound disabled from commandline\n";
  }

  while (true)
  {
    SDL_WM_SetCaption("P Pong by gbr", 0);
    menuShow();
    switch (menuEntry())
    {
      case 'q': cout << "[I] Have a nice day...\n"; SDL_Quit(); return 0;
      case 'i': info(); break;
      case 'g': if (! gioca(true)) msg("FATAL ERROR! New Game: check stderr and stdout!"); break;
      case 'c': if (! gioca(false)) msg("FATAL ERROR! Load Game: check stderr and stdout!"); break;
      case 'o': opzioni();
      default: break;
    }
  }

  SDL_Quit();
  return 0;
}

/*
 * P Pong
 * Programmato da Giovan Battista Rolandi
 * http://gbr.altervista.org
 * Agosto 2012
 *
 * Questo programma è distribuito sotto la
 * Licenza Creative Commons Attribuzione Non Commerciale Condividi allo Stesso Modo 3.0 Italia
 * altrimenti abbreviata Licenza CC BY-NC-SA
*/
